@@ -773,7 +773,7 @@ main (int argc, char *argv[])
                 gtk_window_set_title (GTK_WINDOW (dialog), "");
                 gtk_window_set_icon_name (GTK_WINDOW (dialog), "session-properties");
                 gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-                gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 14);
+                gtk_box_set_spacing (GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog))), 14);
 
                 gtk_dialog_run (GTK_DIALOG (dialog));
                 gtk_widget_destroy (dialog);
